{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit CPortLibU;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ActiveX, AxCtrls, CPortLib_TLB, StdVcl, Menus, CPort, CPortCtl;

type
  TCPortLibX = class(TActiveForm, IDMCPortLibX)
    ComTerminal: TComTerminal;
    ComPort: TComPort;
    PopupMenu: TPopupMenu;
    ActiveItem: TMenuItem;
    SettingsItem: TMenuItem;
    TerminalItem: TMenuItem;
    FontItem: TMenuItem;
    ClearItem: TMenuItem;
    procedure ActiveItemClick(Sender: TObject);
    procedure SettingsItemClick(Sender: TObject);
    procedure TerminalItemClick(Sender: TObject);
    procedure FontItemClick(Sender: TObject);
    procedure ComPortAfterClose(Sender: TObject);
    procedure ComPortAfterOpen(Sender: TObject);
    procedure ClearItemClick(Sender: TObject);
    procedure ComPortRxBuf(Sender: TObject; const Buffer; Count: Integer);
    procedure ComPortRxChar(Sender: TObject; Count: Integer);
  private
    { Private declarations }
    FEvents: IDMCPortLibXEvents;
    FUseWriteStr: boolean;
  protected
    { Protected declarations }
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    function Get_Active: WordBool; safecall;
    procedure Close; safecall;
    procedure Open(Port, BaudRate, Parity, DataBits, StopBits: Integer);
      safecall;
    procedure Write(const Str: WideString); safecall;
    function Get_Visible: WordBool; safecall;
    procedure Set_Visible(Value: WordBool); safecall;
    procedure Set_Active(Value: WordBool); safecall;
  public
    { Public declarations }
    procedure Initialize; override;
end;

implementation

uses ComObj, ComServ;

{$R *.DFM}

{ TCPortLibX }

procedure TCPortLibX.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as IDMCPortLibXEvents;
  inherited EventSinkChanged(EventSink);
end;

procedure TCPortLibX.Initialize;
begin
  inherited Initialize;
  ComPort.OnRxBuf:=ComPortRxBuf; // unable to set at design-time (WHY!!!!!!!?)
end;

function TCPortLibX.Get_Active: WordBool;
begin Result:=ComPort.Connected; end;

procedure TCPortLibX.Set_Active(Value: WordBool);
begin ComPort.Connected:=Value; end;

procedure TCPortLibX.Close;
begin ComPort.Connected:=false; end;

procedure TCPortLibX.Open(Port,BaudRate,Parity,DataBits,StopBits: Integer);
begin
  ComPort.Port:='COM'+IntToStr(Port);
  case BaudRate of
    300:    ComPort.BaudRate:=br300;
    600:    ComPort.BaudRate:=br600;
    1200:   ComPort.BaudRate:=br1200;
    2400:   ComPort.BaudRate:=br2400;
    4800:   ComPort.BaudRate:=br4800;
    9600:   ComPort.BaudRate:=br9600;
    14400:  ComPort.BaudRate:=br14400;
    19200:  ComPort.BaudRate:=br19200;
    38400:  ComPort.BaudRate:=br38400;
    56000:  ComPort.BaudRate:=br56000;
    57600:  ComPort.BaudRate:=br57600;
    115200: ComPort.BaudRate:=br115200;
    128000: ComPort.BaudRate:=br128000;
    256000: ComPort.BaudRate:=br256000;
  else
    ComPort.BaudRate:=br9600;
  end;
  FUseWriteStr:=BaudRate<>300;                               // magic tweak...
  ComPort.Parity.Bits:=TParityBits(Parity);
  ComPort.DataBits:=TDataBits(DataBits-5);                   // databits=5..8!
  ComPort.StopBits:=TStopBits(StopBits);
  ComPort.Connected:=true;
end;

procedure TCPortLibX.Write(const Str: WideString);
var S: shortstring;
begin
  Assert(Length(Str)<254,'TCPortLibX.Write overflow!');
  S:=Str; // using "long" strings produce errors (although debugger values OK)
  if FUseWriteStr then ComPort.WriteStr(S) else ComPort.Write(S, length(S));
end;

function TCPortLibX.Get_Visible: WordBool;
begin Result:=ComTerminal.Visible; end;

procedure TCPortLibX.Set_Visible(Value: WordBool);
begin
  Visible:=Value; ComTerminal.Visible:=Value;
  // not enough to prevent parasitic drawings!
  if Value then ComTerminal.ComPort:=ComPort else ComTerminal.ComPort:=nil;
end;

///////////// COMPONENT EVENTS /////////////////////////

procedure TCPortLibX.ActiveItemClick(Sender: TObject);
begin
  ComPort.Connected:=not ComPort.Connected;
end;

procedure TCPortLibX.SettingsItemClick(Sender: TObject);
begin
  ComPort.ShowSetupDialog;
end;

procedure TCPortLibX.TerminalItemClick(Sender: TObject);
begin
  ComTerminal.ShowSetupDialog;
end;

procedure TCPortLibX.FontItemClick(Sender: TObject);
begin
  ComTerminal.SelectFont;
end;

procedure TCPortLibX.ComPortAfterClose(Sender: TObject);
begin
  ActiveItem.Checked:=ComPort.Connected;
end;

procedure TCPortLibX.ClearItemClick(Sender: TObject);
begin
  ComTerminal.ClearScreen;
end;

procedure TCPortLibX.ComPortAfterOpen(Sender: TObject);
begin
  ActiveItem.Checked:=ComPort.Connected;
end;

////////////// READ EVENT PROCESSING ////////////////////

procedure TCPortLibX.ComPortRxBuf(Sender: TObject; const Buffer;
  Count: Integer);
var Buf: array [0..30000] of char;                      // another limitation!
begin
  Assert(Count<SizeOf(Buf)-1,'TCPortLibX.ComPortRxBuf overflow!');
  Move(Buffer,Buf,Count); Buf[Count]:=#0;
  if FEvents<>nil then FEvents.OnRead(Buf);
end;

procedure TCPortLibX.ComPortRxChar(Sender: TObject; Count: Integer);
var Str: String;
begin
  if Visible then Exit;                    // event fired when terminal is off
  ComPort.ReadStr(Str, Count);
  if FEvents<>nil then FEvents.OnRead(Str);
end;

initialization
  TActiveFormFactory.Create(
    ComServer,
    TActiveFormControl,
    TCPortLibX,
    Class_CPortLibX,
    1,
    '',
    OLEMISC_SIMPLEFRAME or OLEMISC_ACTSLIKELABEL,
    tmApartment);
end.
