{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit HP4191;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  HPIBDev, StdCtrls, Scan, ALed, ExtCtrls, Menus;

type
  THP4191Device = class(THPIBDevice)
    Panel: TPanel;
    Bevel: TBevel;
    ALed: TALed;
    LabelA: TLabel;
    LabelB: TLabel;
    StaticTextA: TStaticText;
    StaticTextA1: TStaticText;
    StaticTextB: TStaticText;
    StaticTextB1: TStaticText;
    ScanFrame: TScanFrame;
    StepStaticText: TStaticText;
    StepEdit: TEdit;
    N2: TMenuItem;
    ModeMenu: TMenuItem;
    AngleMenu: TMenuItem;
    SchemeMenu: TMenuItem;
    ZfiItem: TMenuItem;
    YfiItem: TMenuItem;
    GfiItem: TMenuItem;
    RXItem: TMenuItem;
    GBItem: TMenuItem;
    GxyItem: TMenuItem;
    LItem: TMenuItem;
    CItem: TMenuItem;
    DegreesItem: TMenuItem;
    RadiansItem: TMenuItem;
    RItem: TMenuItem;
    GItem: TMenuItem;
    DItem: TMenuItem;
    QItem: TMenuItem;
    procedure ActiveFormCreate(Sender: TObject);
    procedure ScanFrameSpeedComboBoxChange(Sender: TObject);
    procedure ALedTimer(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure ScanFrameEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ClearItemClick(Sender: TObject);
    procedure TriggerItemClick(Sender: TObject);
    procedure ModeMenuClick(Sender: TObject);
    procedure AngleSchemeMenuClick(Sender: TObject);
  private
    { Private declarations }
    FExtendedOutput: boolean;
    FAutoStop: boolean;
    procedure UpdateMenu;
  protected
    { Protected declarations }
    function Get_Text: WideString; override; safecall;
    procedure Set_Text(const Value: WideString); override; safecall;
    procedure Set_Active(Value: WordBool); override; safecall;
  public
    { Public declarations }
  end;

implementation

uses ActiveX, AxCtrls, ComObj, ComServ, Devices_TLB;

{$R *.DFM}

/////// overridden methods //////

procedure THP4191Device.ActiveFormCreate(Sender: TObject); // additional init.
begin
  inherited; ScanFrame.SpeedComboBox.ItemIndex:=0;
end;

procedure THP4191Device.ClearItemClick(Sender: TObject);
begin
  inherited;
  Set_Text('R0T3EN'); StepEdit.Text:='1.0';   //  Autorange, external trigger!
  with ScanFrame do
  begin
    MinEdit.Text:='1.0'; MaxEdit.Text:='1000.0'; Edit.Text:='1.0';
    UpSpeedButton.Down:=false; DownSpeedButton.Down:=false;
  end;
  ZfiItem.Checked:=true; DegreesItem.Checked:=true; RItem.Checked:=true;
  UpdateMenu;
end;

procedure THP4191Device.TriggerItemClick(Sender: TObject);
begin
  inherited; ALed.Blink:=true; ALed.Value:=true;     // in addition, flash led
end;

function THP4191Device.Get_Text: WideString;
begin                   // return formatted (and buffered) data from displays!
  Result:=StaticTextA1.Caption+'='+StaticTextA.Caption+#13#10+
  StaticTextB1.Caption+'='+StaticTextB.Caption+#13#10+
  ScanFrame.StaticText.Caption+'='+ScanFrame.Edit.Text;
  if FExtendedOutput then Result:=Result+#13#10+'Min='+
  ScanFrame.MinEdit.Text+#13#10+'Max='+ScanFrame.MaxEdit.Text+
  #13#10+'Step='+StepEdit.Text;
end;

procedure THP4191Device.Set_Text(const Value: WideString);
var S: shortstring; I: integer;
begin
  S:=UpperCase(Value);
  if Pos('EXTENDEDOUTPUT',S)>0 then                   // process spec. command
  begin FExtendedOutput:=Pos('ON',S)>0; Exit; end;
  if Pos('AUTOSTOP',S)>0 then begin FAutoStop:=Pos('ON',S)>0; Exit; end;
  if Pos('SCANUP',S)>0 then
  begin ScanFrame.UpSpeedButton.Down:=true; Exit; end;
  if Pos('SCANDOWN',S)>0 then
  begin ScanFrame.DownSpeedButton.Down:=true; Exit; end;
  if Pos('SCANOFF',S)>0 then
  begin
    ScanFrame.UpSpeedButton.Down:=false;
    ScanFrame.DownSpeedButton.Down:=false; Exit;
  end;
  // filter commands and syncronize UI
  if S[1]='A' then ModeMenu.Items[StrToInt(S[2])-1].Checked:=true;
  if S[1]='B' then
  begin
    I:=0; while I<ModeMenu.Count do
    if ModeMenu.Items[I].Checked then Break else Inc(I);
    if I<3 then DegreesItem.Checked:=S[2]='1';
    if I>5 then
    case S[2] of
      '1': RItem.Checked:=true;
      '2': GItem.Checked:=true;
      '3': DItem.Checked:=true;
      '4': QItem.Checked:=true;
    end;
  end;
  UpdateMenu;
  if Pos('FR',S)>0 then ScanFrame.Edit.Text:=Copy(S,3,Pos('EN',S)-3);
  inherited;                                           // write to HPIB driver
end;

procedure THP4191Device.TimerTimer(Sender: TObject);
var Min,Max,Spot,Step: extended; S, L: shortstring;
begin
  Timer.Enabled:=false;
  S:=inherited Get_Text;                  // NOTE!!! WE CALL INHERITED METHOD!
  if S[2]=' ' then Delete(S,2,1);                       // cut parasitic space
  if (S[2]='O') or (S[17]='O') then MessageBeep($ffffffff);  // check overflow
  case S[3] of
    'Z': L:='|Z|  Ohm';
    'Y': L:='|Y|     Sm';
    'M': L:='||';
    'R': L:='R  Ohm';
    'G': L:='G     Sm';
    'X': L:='x';
    'L': L:='L     H';
    'C': L:='C     F';
  else L:='?   ???';
  end;
  StaticTextA1.Caption:=L; StaticTextA.Caption:=Copy(S, 5, 11);
  case S[18] of
    'D': if S[3] in ['Z','Y','M'] then L:='Fi Deg' else L:='D';
    'R': if S[3] in ['Z','Y','M'] then L:='Fi Rad' else L:='R Ohm';
    'X': L:='X  Ohm';
    'B': L:='B     Sm';
    'Y': L:='y';
    'G': L:='G     Sm';
    'Q': L:='Q';
  else L:='?  ???';
  end;
  StaticTextB1.Caption:=L; StaticTextB.Caption:=Copy(S, 20, 11);
  

  if FEvents<>nil then FEvents.OnData;


  try                                                                  // scan
    Min:=StrToFloat(ScanFrame.MinEdit.Text);
    Max:=StrToFloat(ScanFrame.MaxEdit.Text);
    Spot:=StrToFloat(ScanFrame.Edit.Text);
    Step:=StrToFloat(StepEdit.Text);
    if ScanFrame.UpSpeedButton.Down then
      if (Spot+Step<=Max) then
      begin
        L:=FloatToStrF(Spot+Step,ffFixed,10,4); Set_Text('FR'+L+'EN');
      end else if FAutoStop then InternalTriggerItem.Checked:=false;
    if ScanFrame.DownSpeedButton.Down then
      if (Spot-Step>=Min) then
      begin
        L:=FloatToStrF(Spot-Step,ffFixed,10,4); Set_Text('FR'+L+'EN');
      end else if FAutoStop then InternalTriggerItem.Checked:=false;
  except
    InternalTriggerItem.Checked:=false;            // disable internal trigger
  end;

  if InternalTriggerItem.Checked then TriggerItem.Click;

end;

procedure THP4191Device.Set_Active(Value: WordBool);
  procedure doEnable(C: TControl);
  var I: integer;
  begin
    for I:=0 to C.ComponentCount-1 do
    if C.Components[I] is TControl
    then (C.Components[I] as TControl).Enabled:=Value;
  end;
begin
  inherited; doEnable(Self); doEnable(ScanFrame); UpdateMenu;
end;

//// other stuff /////////

procedure THP4191Device.ALedTimer(Sender: TObject);                // hide led
begin ALed.Blink:=false; ALed.Value:=false; end;

procedure THP4191Device.ScanFrameSpeedComboBoxChange(Sender: TObject);
begin
  case ScanFrame.SpeedComboBox.ItemIndex of
    0: Timer.Interval:=1000;
    1: Timer.Interval:=3000;
    2: Timer.Interval:=10000;
  end;
end;

procedure THP4191Device.ScanFrameEditKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  inherited;
  if Key=VK_RETURN then Set_Text('FR'+ScanFrame.Edit.Text+'EN');
end;

procedure THP4191Device.UpdateMenu;
var I: integer;
begin
  I:=0; while I<ModeMenu.Count do
  if ModeMenu.Items[I].Checked then Break else Inc(I);
  ModeMenu.Enabled:=ActiveItem.Checked;
  AngleMenu.Enabled:=(I<3) and ActiveItem.Checked;
  SchemeMenu.Enabled:=(I>5) and ActiveItem.Checked;
end;

procedure THP4191Device.ModeMenuClick(Sender: TObject);
begin Set_Text('A'+IntToStr((Sender as TMenuItem).Tag)+'EN'); end;

procedure THP4191Device.AngleSchemeMenuClick(Sender: TObject);
begin Set_Text('B'+IntToStr((Sender as TMenuItem).Tag)+'EN'); end;

initialization
  TActiveFormFactory.Create(
    ComServer,
    TActiveFormControl,
    THP4191Device,
    CLASS_HP4191X,
    2,
    '',
    OLEMISC_SIMPLEFRAME or OLEMISC_ACTSLIKELABEL,
    tmApartment);
end.
