{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit HPIBDev;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ActiveX, AxCtrls, Devices_TLB, StdVcl, Menus, ExtCtrls, StdCtrls;

type
  THPIBDevice = class(TActiveForm, IDMDevice)
    PopupMenu: TPopupMenu;
    InternalTriggerItem: TMenuItem;
    LocalControlItem: TMenuItem;
    ClearItem: TMenuItem;
    N1: TMenuItem;
    ActiveItem: TMenuItem;
    TriggerItem: TMenuItem;
    Timer: TTimer;
    procedure InternalTriggerItemClick(Sender: TObject);
    procedure LocalControlItemClick(Sender: TObject);
    procedure ClearItemClick(Sender: TObject);
    procedure ActiveItemClick(Sender: TObject);
    procedure TriggerItemClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure ActiveFormDestroy(Sender: TObject);
  private
    { Private declarations }
    FAddress: integer;                                  // HPIB device address
    FFormCreated: boolean;                             // need for CreateEvent
    function CheckHPIB: boolean;                       // true if no HW access
    procedure ActiveFormShow(Sender: TObject);
  protected
    { Protected declarations }
    FEvents: IDMDeviceEvents;
    procedure DefinePropertyPages(DefinePropertyPage:
      TDefinePropertyPage); override;
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    function Get_InternalTrigger: WordBool; safecall;
    function Get_Port: Integer; safecall;
    function Get_Active: WordBool; safecall;
    procedure LoadConfig; virtual; safecall;      // next 7 methods  should be
    procedure SaveConfig; virtual; safecall;      // overridden in descendants
    function Get_Text: WideString; virtual; safecall;
    procedure Set_Active(Value: WordBool); virtual; safecall;
    procedure Set_Text(const Value: WideString); virtual; safecall;
    procedure Set_InternalTrigger(Value: WordBool); virtual; safecall;
    procedure Set_Port(Value: Integer); virtual; safecall; // may display port
    procedure Trigger; safecall;                     // call menu item handler
    procedure Clear; safecall;                       // call menu item handler
  public
    { Public declarations }
    procedure Initialize; override;
  end;

implementation

uses ComObj, ComServ, HPIBDrv;

{$R *.DFM}

var ISC: word=7;

// WARNING!!! Descendants NEVER should have direct access to HPIB driver!!!

{ THPIBDevice }

// ActiveX service methods - do not modify!

procedure THPIBDevice.DefinePropertyPages(DefinePropertyPage:
  TDefinePropertyPage);
begin
  { Define property pages here.  Property pages are defined by calling
    DefinePropertyPage with the class id of the page.  For example,
    DefinePropertyPage(Class_HPIBDevicePage); }
end;

procedure THPIBDevice.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as IDMDeviceEvents;
end;

{ NOTE: we can't move code from ActiveFormShow() to EventSinkChanged() and
remove Initialize() and ActiveFormShow() methods - in this case OnCreate()
event invoked only for one device of two! Why? Nobody knows... }

procedure THPIBDevice.Initialize;
begin
  inherited Initialize;
  OnShow:=ActiveFormShow;
end;

procedure THPIBDevice.ActiveFormShow(Sender: TObject);
begin
  if (FEvents<>nil) and (not FFormCreated) then
  begin FEvents.OnCreate; FFormCreated:=true; end;
end;

procedure THPIBDevice.ActiveFormDestroy(Sender: TObject);
begin if FEvents<>nil then FEvents.OnDestroy; end;

//////// Utility /////////

function THPIBDevice.CheckHPIB: boolean;
begin
  Result:=not ActiveItem.Checked;
  if not DriverOpened then
  begin
    HpibOpen(ISC); ErrorHandler(hpibReset(ISC));
    ErrorHandler(hpibTimeOut(ISC,3));
  end;
end;

/////// Interface methods /////////

// next 3 static methods always do the same job in all devices!

function THPIBDevice.Get_Active: WordBool;
begin Result:=ActiveItem.Checked; end;

function THPIBDevice.Get_Port: Integer;
begin Result:=FAddress; end;

function THPIBDevice.Get_InternalTrigger: WordBool;
begin Result:=InternalTriggerItem.Checked; end;

// Next 7 methods probably will be overridden in descendants

procedure THPIBDevice.Set_Active(Value: WordBool);
begin                                      // may also display Activity status
  ActiveItem.Checked:=Value;
  LocalControlItem.Enabled:=Value;             // disable commands if inactive
  ClearItem.Enabled:=Value;
  TriggerItem.Enabled:=Value;
  InternalTriggerItem.Enabled:=Value;
  if not Value then                            // also CLEAR internal trigger!
  begin
    InternalTriggerItem.Checked:=false; Timer.Enabled:=false;
  end;
end;

procedure THPIBDevice.Set_Port(Value: Integer);      // may display Port value
begin FAddress:=Value; end;

procedure THPIBDevice.Set_InternalTrigger(Value: WordBool);
begin                               // also may display InternalTrigger status
  if ActiveItem.Checked then                  // NOTE: do nothing if inactive!
  begin InternalTriggerItem.Checked:=Value; if Value then Trigger; end;
end;

function THPIBDevice.Get_Text: WideString;
var S: string[250]; I: word;                    // NOTE: S NOT largestring!!!!
begin                             // always overridden to return buffered data
  S:='';                                  // by default, read from HPIB driver
  if not CheckHPIB then                    // return empty string if inactive!
  begin
    for I:=1 to 250 do S:=S+' '; I:=length(S)-1;
    ErrorHandler(hpibEnters(ISC*100+FAddress, S[1], I));
  end;
  Result:=S;
end;

procedure THPIBDevice.Set_Text(const Value: WideString);
var S: string[250]; b: byte;             // always overridden to syncronize UI
begin                                      // by default, write to HPIB driver
  if CheckHPIB then Exit;                     // NOTE: do nothing if inactive!
  S:=Value; // warning: usage of openstrings in thunk causes Delphi MM crash!!
  if (Length(S)=3) and (S[1]='$') then  // special case: using hpibOutputab()!
  begin
    b:=StrToInt(S);
    ErrorHandler(hpibOutputab(ISC*100+FAddress, b, 1, 1));
  end else
  ErrorHandler(hpibOutputs(ISC*100+FAddress, S[1], length(S)));
end;

procedure THPIBDevice.LoadConfig;                                     // empty
begin end;

procedure THPIBDevice.SaveConfig;                                     // empty
begin end;

////// Next 2 static methods just call menu item handler

procedure THPIBDevice.Trigger;
begin TriggerItem.Click; end;

procedure THPIBDevice.Clear;
begin ClearItem.Click; end;

/////// menu and timer handlers ////////////

// these 4 handlers may be changed in descendants

procedure THPIBDevice.LocalControlItemClick(Sender: TObject);
begin
  if CheckHPIB then Exit;
  if FAddress<>0 then
  ErrorHandler(hpibLocal(ISC*100+FAddress))
  else ErrorHandler(hpibLocal(ISC));
end;

procedure THPIBDevice.ClearItemClick(Sender: TObject);
begin
  if CheckHPIB then Exit;
  if FAddress<>0 then
  ErrorHandler(hpibClear(ISC*100+FAddress))
  else ErrorHandler(hpibClear(ISC));
end;

procedure THPIBDevice.TriggerItemClick(Sender: TObject);
begin
  if CheckHPIB then Exit;
  if FAddress<>0 then
  ErrorHandler(hpibTrigger(ISC*100+FAddress))
  else ErrorHandler(hpibTrigger(ISC));
  Timer.Enabled:=true;
end;

{ these 2 handlers NEVER should be overridden! Instead, override virtual
Set_InternalTrigger() and Set_Active() methods. Obviously we can't just
override handlers like in Trigger() and Clear() because Set_XXX methods
have parameter! }

procedure THPIBDevice.InternalTriggerItemClick(Sender: TObject);
begin Set_InternalTrigger(not InternalTriggerItem.Checked); end;

procedure THPIBDevice.ActiveItemClick(Sender: TObject);
begin Set_Active(not ActiveItem.Checked); end;

///// NOTE: TimerTimer() ALWAYS must be overridden to display device data!!!

procedure THPIBDevice.TimerTimer(Sender: TObject);
begin
  Timer.Enabled:=false;
  if FEvents<>nil then FEvents.OnData;
  if InternalTriggerItem.Checked then TriggerItem.Click;
end;

end.
