{****************************************************************************}
{                            Data Master 2000                                }
{****************************************************************************}
unit HPIBDrv;
{$B-}
interface

function DriverOpened: boolean;                     // used instead of HPIBLib

function HpibLocal(SC: LONGINT): word;                      // main driver API
function HpibTrigger(SC: LONGINT): word;
function HpibTimeout(SC: LONGINT; P1: DOUBLE): word;
function HpibClear(SC: LONGINT): word;
function HpibClose: word;
function HpibOpen(SC: LONGINT): word;
function HpibReset(SC: LONGINT): word;
function HpibEnters(SC: LONGINT; VAR Data; VAR ByteCount: word): word;
function HpibOutputs(SC: LONGINT; VAR Data; ByteCount: word): word;
function HpibOutputab(SC: LONGINT; VAR P1; P2,P3: word): word;
procedure ErrorHandler(ErrCode: integer);

implementation

uses Windows, Forms, Controls, StdCtrls, SysUtils;

type EHardwareError=class(Exception);

/////////////// driver - specific code ///////////////////////////////////////
// This unit first tries to get pointers to W9x specific kernel calls. If
// it fails (W2K/NT!), it set LibHandler=0 (debug mode).
// Second, it tries to load 16-bit marshaler to HPIB.DLL using LoadLibrary16()
// If this call fails, debug mode will be used again.
// Note that you can modify this code to add support for NEW cards!

resourcestring
  strThunkError='Unable to obtain function <%s>!';
  strHPIBError='Critical HPIB driver error:'#10'%s';
  hpibError1='Unknown HPIB error!';            // hpib card error descriptions
  hpibError2='Invalid HPIB select code or device address!';
  hpibError3='HPIB value out of range!';
  hpibError4='HPIB timeout!';
  hpibError5='HPIB must be controller!';
  hpibError6='HPIB pass control not permitted!';
  hpibError7='Invalid HPIB number!';
  hpibError8='Improper HPIB addressing!';
  hpibError9='File I/O HPIB error!';
  hpibError10='Cannot open this HPIB card!';
  hpibError11='HPIB card is not open!';
  hpibError12='Cannot close HPIB card!';
  hpibError13='Invalid handle for this HPIB ISC!';

var
  pFunc: pointer; FuName: array[0..99] of char;          // need to call thunk
  LibHandler: THandle=0;                          // need for HPIB card driver
  LibOpened: boolean=false;                // incapsulated with DriverOpened()
  TermF: TForm=nil; TermM: TMemo=nil;                // used by debug terminal

// Undocumented Kernel32 calls (only for W9x)
// Note: we use DYNAMIC binding!

QT_Thunk: procedure; cdecl;
{$ifdef safe_w2k}                             // this version fails under W98!
LoadLibrary16: function(LibraryName: PChar): THandle; stdcall;
FreeLibrary16: procedure(HInstance: THandle); stdcall;
GetProcAddress16: function(HInstance: THandle;
  ProcName: PChar): Pointer; stdcall;
{$else}
function LoadLibrary16(LibraryName: PChar): THandle; stdcall;
external kernel32 index 35;
procedure FreeLibrary16(HInstance: THandle); stdcall;
external kernel32 index 36;
function GetProcAddress16(HInstance: THandle;
  ProcName: PChar): Pointer; stdcall;
external kernel32 index 37;
{$endif}

function CallProc16(a,b: word): word;                       {implements thunk}
var ThunkTrash: array[0..$20] of Word;
begin                            {in LibHandler must be valid library handler}
  ThunkTrash[0]:=LibHandler;           {in FuName - name of func to be called}
  pFunc:=GetProcAddress16(LibHandler, FuName);
  if pFunc=nil then raise Exception.CreateFmt(strThunkError,[FuName]);
  asm
    push a                                                 { push argument 1 }
    push b                                                 { push argument 2 }
    mov edx, pFunc                           { load 16-bit procedure pointer }
    call QT_Thunk                                               { call thunk }
    mov Result, ax                                         { save the result }
  end;
end;

///// utility routines /////

function DriverOpened: boolean;
begin Result:=LibOpened; end;

procedure AddLine(S: string);    {correctly displays string in debug terminal}
begin
  if Assigned(TermF) and Assigned(TermM) then
  begin if TermM.Lines.Count>50 then TermM.Clear; TermM.Lines.Add(S); end;
end;

//////////////////////////////////////////////////////////////////////////////
// Note: LibHandler=0 means DEBUG MODE!!!
// All HpibXXX() procedures should be checked: ErrorHandler(HpibXXX(...))

function HpibLocal(SC: LONGINT): word;
begin
  if LibHandler=0
  then begin Result:=0; AddLine('Set local address '+IntToStr(SC)); end
  else begin FuName:='LocalHPIB'; Result:=CallProc16(sc,1); end;
end;

function HpibTrigger(SC: LONGINT): word;
begin
  if LibHandler=0
  then begin Result:=0; AddLine('Trigger address '+IntToStr(SC)); end
  else begin FuName:='TriggerHPIB'; Result:=CallProc16(sc,1); end;
end;

function HpibTimeout(SC: LONGINT; P1: DOUBLE): word;
begin
  if LibHandler=0 then
  begin
    Result:=0;
    AddLine('Set timeout '+FloatToStr(P1)+' for card '+IntToStr(SC));
  end else
  begin FuName:='TimeoutHPIB'; Result:=CallProc16(round(P1*1000{!}),sc); end;
end;

function HpibClear(SC: LONGINT): word;
begin
  if LibHandler=0
  then begin Result:=0; AddLine('Cleared address '+IntToStr(SC)); end
  else begin FuName:='ClearHPIB'; Result:=CallProc16(sc,1); end;
end;

function HpibClose: word;
begin
  LibOpened:=false;
  if LibHandler=0
  then begin Result:=0; AddLine('Library closed'); end
  else begin FuName:='CloseHPIB'; Result:=CallProc16(1,1); end;
end;

function HpibOpen(SC: LONGINT): word;
begin
  LibOpened:=true;
  if LibHandler=0
  then begin Result:=0; AddLine('Card #'+IntToStr(SC)+' opened'); end
  else begin FuName:='OpenHPIB'; Result:=CallProc16(sc,1); end;
end;

function HpibReset(SC: LONGINT): word;
begin
  if LibHandler=0
  then begin Result:=0; AddLine('Reset address '+IntToStr(SC)); end
  else begin FuName:='ResetHPIB'; Result:=CallProc16(sc,1); end;
end;

function HpibEnters(SC: LONGINT; VAR Data; VAR ByteCount: word): word;
var I: word; A: array[1..250] of byte absolute Data; S: string[200];
begin
  if LibHandler=0 then
  begin
    case SC of
     703: S:=Format('2.%d723E+1 D',[Random(9)]);
     707: S:=Format('R +1135%dE-4',[Random(9)]);
     717: S:=Format(' NRN+1.23%d5E+01,NXN+5.432%dE-01',[Random(9),Random(5)]);
    else  S:='DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG';
    end;
    Result:=0; AddLine(Format('Read from %d: %s',[SC,S]));
    for I:=1 to length(S) do if I<ByteCount then A[I]:=byte(S[I]);
  end else
  begin
    FuName:='EntersHPIB'; Result:=CallProc16(sc,1);{perform real hpib reading}
    if Result<>0 then Exit;                        {HPIB.DLL returns errcode!}
    FuName:='GetBufLen'; ByteCount:=CallProc16(1,1);  {obtain number of chars}
    I:=1; FuName:='GetBuf';
    while (I<=ByteCount) and (I<250) do                           {read chars}
    begin
      A[I]:=Lo(CallProc16(I,1)); Inc(I);
    end;
  end;
end;

function HpibOutputs(SC: LONGINT; VAR Data; ByteCount: word): word;
var I: word; A: array[1..250] of byte absolute Data; S: string[200];
begin
  if LibHandler=0 then
  begin
    S:='                                                           ';
    for I:=1 to ByteCount do if ByteCount<length(S) then S[I]:=char(A[I]);
    Result:=0; AddLine(Format('Write to %d: %s',[SC,S]));
  end else
  begin
    FuName:='SetBufLen'; CallProc16(ByteCount,1);          {set buffer length}
    I:=1; FuName:='SetBuf';
    while (I<=ByteCount) and (I<250) do                          {write chars}
    begin
      CallProc16(I,A[I]); Inc(I);
    end;
    FuName:='OutputsHPIB'; Result:=CallProc16(sc,1);    {perform real writing}
  end;
end;

function HpibOutputab(SC: LONGINT; VAR P1; P2,P3: word): word;
{arbitrary array of bytes??? P2-size, P3-"swap"?????}
var I: word; A: array[1..250] of byte absolute P1;
begin
  if LibHandler=0 then
  begin
    Result:=0; AddLine('Write bytes to '+IntToStr(SC));
  end else
  begin
    FuName:='SetBufLen'; CallProc16(P2,1);       {just same as HpibOutputs...}
    I:=1; FuName:='SetBuf';
    while (I<=P2) and (I<250) do
    begin
      CallProc16(I,A[I]); Inc(I);
    end;
    FuName:='OutputABHPIB'; Result:=CallProc16(sc,1);
  end;
end;

procedure ErrorHandler(ErrCode: integer);
var s: string;
begin
  if ErrCode<>0 then                                             {0 means OK!}
  begin
    HpibClose;                             {resume library as in tpwmeter.pas}
    case ErrCode of
      1:  S:=hpibError1;
      2:  S:=hpibError2;
      3:  S:=hpibError3;
      4:  S:=hpibError4;
      5:  S:=hpibError5;
      6:  S:=hpibError6;
      7:  S:=hpibError7;
      8:  S:=hpibError8;
      9:  S:=hpibError9;
      10: S:=hpibError10;
      11: S:=hpibError11;
      12: S:=hpibError12;
      13: S:=hpibError13;
    else S:='?????';
    end;
    raise EHardwareError.CreateFmt(strHPIBError,[S]);
  end;
end;

//////////////////////////////////////////////////////////////////////////////
var FN: array[0..MAX_PATH] of char;

initialization
  LibHandler:=LoadLibrary('kernel32.dll');
  QT_Thunk:=GetProcAddress(LibHandler,'QT_Thunk');
  {$ifdef safe_w2k}
  LoadLibrary16:=GetProcAddress(LibHandler,MakeIntResource(35));
  FreeLibrary16:=GetProcAddress(LibHandler,MakeIntResource(36));
  GetProcAddress16:=GetProcAddress(LibHandler,MakeIntResource(37));
  {$endif}
  FreeLibrary(LibHandler); LibHandler:=0;
  if Assigned(QT_Thunk)
  {$ifdef safe_w2k}
     and Assigned(LoadLibrary16) and Assigned(FreeLibrary16) and
     Assigned(GetProcAddress16)
  {$endif}
  then begin
    GetModuleFilename(hInstance, FN, MAX_PATH);
    StrPCopy(FN,ExtractFilePath(StrPas(FN))+'HPIB_THK.DLL');
    LibHandler:=LoadLibrary16(FN);
  end;
  if LibHandler<=32 then
  begin
    LibHandler:=0;
    TermF:=TForm.CreateNew(nil);
    TermF.Caption:='HPIB driver debug terminal'; TermF.BorderIcons:=[];
    TermM:=TMemo.Create(TermF); TermM.Parent:=TermF; TermM.Align:=alClient;       {create memo}
    TermF.Show;
  end;
finalization
  if LibHandler>=32 then FreeLibrary16(LibHandler);
  if Assigned(TermF) then TermF.Free;
end.
